const path = require('path');

const through = require('through2');
const PluginError = require('plugin-error');

const handle = require('./lib/handle');

global.PLUGIN_NAME = 'gulp-image-handler';

module.exports = (option = {}) => through.obj(async function (file, enc, callback) {
    try {
        if (file.isStream()) {
            this.emit('error', new PluginError(PLUGIN_NAME, 'Streams are not supported.'));
            return callback();
        }

        if (file.isBuffer()) {
            let acceptSuffixs = ['.png', '.jpeg', '.gif', '.jpg'];
            let acceptedFiles = [];

            let contentStr = file.contents.toString();
            let reg = /url\((?![\'\"]?http[s]?:\/\/)[\'\"]?(.*?)[\'\"]?\)/img;
            contentStr.replace(reg, (match, originPath) => {
                if (acceptSuffixs.includes(path.extname(originPath))) {
                    acceptedFiles.push({
                        originPath,
                        absolutePath: path.resolve(path.dirname(file.path), originPath)
                    });
                }
            });

            let uploadFiles = Array.from(new Set(acceptedFiles.map(file => file.absolutePath)));
            let responses = await Promise.all(uploadFiles.map((file) => handle(file, option)));
            responses.filter(response => response && response.absolutePath)
                .forEach((response) => {
                    contentStr = acceptedFiles
                        .filter((file) => file.absolutePath === response.absolutePath)
                        .reduce((contentStr, file) => {
                            return response.content ?
                                contentStr.replace(file.originPath, response.content) :
                                contentStr;
                        }, contentStr)
                });
            file.contents = Buffer.from(contentStr);
        }
    } catch (e) {
        console.error(e.message || e)
    } finally {
        this.push(file);

        callback();
    }

});