const fs = require('fs');
const path = require('path');

const PluginError = require('plugin-error');
const request = require('request');
const mime = require("mime");
const log = require('fancy-log');

const url = 'http://frs.58corp.com/upload';

function _2base64(absolutePath) {
    return new Promise((resolve, reject) => {
        let mimetype = mime.getType(absolutePath);
        let base64 = fs.readFileSync(absolutePath, 'base64');
        resolve({
            absolutePath,
            content: `data:${mimetype ? mimetype + ";" : ""}base64,${base64}`
        });
    })
}

function _upload(absolutePath, compress) {
    return new Promise((resolve, reject) => {
        let form = request.post(url, {
                timeout: 10000
            }, (err, res) => {
                if (err || !res) {
                    reject(err);
                } else {
                    try {
                        let body = JSON.parse(res.body);
                        if (body.statusCode === 0 && body.data[0] && body.data[0].url) {
                            resolve({
                                absolutePath,
                                content: body.data[0].url
                            });
                        } else {
                            throw Error('Server Error')
                        }
                    } catch (e) {
                        log.error(`\nFailed\t${absolutePath}`);
                        reject(e);
                    }
                }
            })
            .form();
        form.append('compress', compress ? 'true' : '');
        form.append('creator', 'Gulp Plugin');
        form.append('platform', 'Gulp Plugin');
        form.append('files', fs.createReadStream(absolutePath));
    });
}

module.exports = (absolutePath, {
    compress = true,
    threshold = 1024
} = {}) => {
    if (fs.existsSync(absolutePath)) {
        if (fs.statSync(absolutePath).size <= ~~threshold) {
            return _2base64(absolutePath);
        } else {
            return _upload(absolutePath, compress);
        }
    }
}